/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.aries.blueprint.plugin.config;

import java.lang.reflect.AnnotatedElement;

import org.apache.aries.blueprint.annotation.config.Config;
import org.apache.aries.blueprint.plugin.spi.BeanAnnotationHandler;
import org.apache.aries.blueprint.plugin.spi.BeanEnricher;
import org.apache.aries.blueprint.plugin.spi.ContextEnricher;
import org.apache.aries.blueprint.plugin.spi.XmlWriter;

public class ConfigAnnotationHandler implements BeanAnnotationHandler<Config>{

    @Override
    public Class<Config> getAnnotation() {
        return Config.class;
    }

    @Override
    public void handleBeanAnnotation(AnnotatedElement annotatedElement, String id,
                                     ContextEnricher contextEnricher, BeanEnricher beanEnricher) {
        Config config = annotatedElement.getAnnotation(Config.class);
        XmlWriter writer = new ConfigWriter(config);
        contextEnricher.addBlueprintContentWriter("config", writer);
        contextEnricher.getBlueprintConfiguration().getNamespaces().add(ConfigWriter.CONFIG_NS);
        
    }

}
